"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExportedNamesOfSource = exports.hasModifier = exports.evaluate = void 0;
const typescript_1 = __importDefault(require("typescript"));
function evaluate(expr, known) {
    if (typescript_1.default.isPrefixUnaryExpression(expr)) {
        const value = evaluate(expr.operand, known);
        if (typeof value === 'number') {
            switch (expr.operator) {
                case typescript_1.default.SyntaxKind.PlusToken:
                    return value;
                case typescript_1.default.SyntaxKind.MinusToken:
                    return -value;
                case typescript_1.default.SyntaxKind.TildeToken:
                    return ~value;
            }
        }
    }
    else if (typescript_1.default.isBinaryExpression(expr)) {
        const left = evaluate(expr.left, known);
        const right = evaluate(expr.right, known);
        if (typeof left === 'number' && typeof right === 'number') {
            switch (expr.operatorToken.kind) {
                case typescript_1.default.SyntaxKind.BarToken:
                    return left | right;
                case typescript_1.default.SyntaxKind.AmpersandToken:
                    return left & right;
                case typescript_1.default.SyntaxKind.GreaterThanGreaterThanToken:
                    return left >> right;
                case typescript_1.default.SyntaxKind.GreaterThanGreaterThanGreaterThanToken:
                    return left >>> right;
                case typescript_1.default.SyntaxKind.LessThanLessThanToken:
                    return left << right;
                case typescript_1.default.SyntaxKind.CaretToken:
                    return left ^ right;
                case typescript_1.default.SyntaxKind.AsteriskToken:
                    return left * right;
                case typescript_1.default.SyntaxKind.SlashToken:
                    return left / right;
                case typescript_1.default.SyntaxKind.PlusToken:
                    return left + right;
                case typescript_1.default.SyntaxKind.MinusToken:
                    return left - right;
                case typescript_1.default.SyntaxKind.PercentToken:
                    return left % right;
                case typescript_1.default.SyntaxKind.AsteriskAsteriskToken:
                    return Math.pow(left, right);
            }
        }
        else if (typeof left === 'string'
            && typeof right === 'string'
            && expr.operatorToken.kind === typescript_1.default.SyntaxKind.PlusToken) {
            return left + right;
        }
    }
    else if (typescript_1.default.isStringLiteralLike(expr)) {
        return expr.text;
    }
    else if (typescript_1.default.isNumericLiteral(expr)) {
        return +expr.text;
    }
    else if (typescript_1.default.isParenthesizedExpression(expr)) {
        return evaluate(expr.expression, known);
    }
    else if (typescript_1.default.isIdentifier(expr)) {
        const value = known.get(expr.text);
        if (value === undefined) {
            throw new Error('unsupported enum. must reference self value');
        }
        return value;
    }
    throw new Error('unexpected evaluation for enum member: ' + expr.getText);
}
exports.evaluate = evaluate;
function hasModifier(node, modifier) {
    var _a;
    return ((_a = node.modifiers) === null || _a === void 0 ? void 0 : _a.some((mod) => mod.kind === modifier));
}
exports.hasModifier = hasModifier;
const cachedNames = new WeakMap();
function getExportedNamesOfSource(program, sourceFile) {
    const cached = cachedNames.get(sourceFile);
    if (cached)
        return cached;
    const typeChecker = program.getTypeChecker();
    const sourceSymbol = typeChecker.getSymbolAtLocation(sourceFile);
    let names;
    if (sourceSymbol) {
        names = typeChecker.getExportsOfModule(sourceSymbol).map(s => {
            if (s.flags & typescript_1.default.SymbolFlags.Alias) {
                return typeChecker.getAliasedSymbol(s).name;
            }
            return s.name;
        });
    }
    else {
        names = [];
    }
    cachedNames.set(sourceFile, names);
    return names;
}
exports.getExportedNamesOfSource = getExportedNamesOfSource;
