import ts from 'typescript';
import { evaluate, getExportedNamesOfSource, hasModifier } from './utils';
export default function (program, pluginOptions) {
    return (ctx) => {
        return (sourceFile) => {
            return ts.visitEachChild(sourceFile, visitor, ctx);
            function visitor(node) {
                if (!ts.isEnumDeclaration(node)) {
                    return ts.visitEachChild(node, visitor, ctx);
                }
                if (!hasModifier(node, ts.SyntaxKind.ConstKeyword)) {
                    return node;
                }
                if (!hasModifier(node, ts.SyntaxKind.ExportKeyword)) {
                    const exportedNames = getExportedNamesOfSource(program, sourceFile);
                    if (!exportedNames.includes(node.name.text)) {
                        return node;
                    }
                }
                if (sourceFile.isDeclarationFile) {
                    return ts.visitEachChild(node, stripConstKeyword, ctx);
                }
                return transformEnum(node);
            }
        };
    };
    function stripConstKeyword(node) {
        return node.kind === ts.SyntaxKind.ConstKeyword ? undefined : node;
    }
    function transformEnum(node) {
        const members = node.members;
        const known = new Map();
        const properties = [];
        let lastValue = -1;
        for (const member of members) {
            if (!ts.isIdentifier(member.name)) {
                throw new Error('The name of enum member must be an identifier');
            }
            const name = member.name.getText();
            let value;
            let valueExpr;
            if (member.initializer) {
                value = evaluate(member.initializer, known);
                if (typeof value === 'number') {
                    valueExpr = ts.factory.createNumericLiteral(value);
                    lastValue = value;
                }
                else if (typeof value === 'string') {
                    valueExpr = ts.factory.createStringLiteral(value);
                    lastValue = value;
                }
                else
                    throw new Error('Unsupported member value');
            }
            else {
                if (typeof lastValue === 'string') {
                    throw new Error('Invalid enum');
                }
                value = ++lastValue;
                valueExpr = ts.factory.createNumericLiteral(value);
            }
            const assignment = ts.factory.createPropertyAssignment(name, valueExpr);
            ts.setSourceMapRange(assignment, ts.getSourceMapRange(member));
            properties.push(assignment);
            known.set(name, value);
        }
        const result = ts.factory.createVariableStatement(node.modifiers, ts.factory.createVariableDeclarationList([
            ts.factory.createVariableDeclaration(node.name, undefined, undefined, ts.factory.createAsExpression(ts.factory.createObjectLiteralExpression(properties, true), ts.factory.createTypeReferenceNode(ts.factory.createIdentifier('const'), undefined))),
        ], ts.NodeFlags.Const));
        ts.setSourceMapRange(result, ts.getSourceMapRange(node));
        return result;
    }
}
