import ts from 'typescript';
export function evaluate(expr, known) {
    if (ts.isPrefixUnaryExpression(expr)) {
        const value = evaluate(expr.operand, known);
        if (typeof value === 'number') {
            switch (expr.operator) {
                case ts.SyntaxKind.PlusToken:
                    return value;
                case ts.SyntaxKind.MinusToken:
                    return -value;
                case ts.SyntaxKind.TildeToken:
                    return ~value;
            }
        }
    }
    else if (ts.isBinaryExpression(expr)) {
        const left = evaluate(expr.left, known);
        const right = evaluate(expr.right, known);
        if (typeof left === 'number' && typeof right === 'number') {
            switch (expr.operatorToken.kind) {
                case ts.SyntaxKind.BarToken:
                    return left | right;
                case ts.SyntaxKind.AmpersandToken:
                    return left & right;
                case ts.SyntaxKind.GreaterThanGreaterThanToken:
                    return left >> right;
                case ts.SyntaxKind.GreaterThanGreaterThanGreaterThanToken:
                    return left >>> right;
                case ts.SyntaxKind.LessThanLessThanToken:
                    return left << right;
                case ts.SyntaxKind.CaretToken:
                    return left ^ right;
                case ts.SyntaxKind.AsteriskToken:
                    return left * right;
                case ts.SyntaxKind.SlashToken:
                    return left / right;
                case ts.SyntaxKind.PlusToken:
                    return left + right;
                case ts.SyntaxKind.MinusToken:
                    return left - right;
                case ts.SyntaxKind.PercentToken:
                    return left % right;
                case ts.SyntaxKind.AsteriskAsteriskToken:
                    return Math.pow(left, right);
            }
        }
        else if (typeof left === 'string'
            && typeof right === 'string'
            && expr.operatorToken.kind === ts.SyntaxKind.PlusToken) {
            return left + right;
        }
    }
    else if (ts.isStringLiteralLike(expr)) {
        return expr.text;
    }
    else if (ts.isNumericLiteral(expr)) {
        return +expr.text;
    }
    else if (ts.isParenthesizedExpression(expr)) {
        return evaluate(expr.expression, known);
    }
    else if (ts.isIdentifier(expr)) {
        const value = known.get(expr.text);
        if (value === undefined) {
            throw new Error('unsupported enum. must reference self value');
        }
        return value;
    }
    throw new Error('unexpected evaluation for enum member: ' + expr.getText);
}
export function hasModifier(node, modifier) {
    var _a;
    return ((_a = node.modifiers) === null || _a === void 0 ? void 0 : _a.some((mod) => mod.kind === modifier));
}
const cachedNames = new WeakMap();
export function getExportedNamesOfSource(program, sourceFile) {
    const cached = cachedNames.get(sourceFile);
    if (cached)
        return cached;
    const typeChecker = program.getTypeChecker();
    const sourceSymbol = typeChecker.getSymbolAtLocation(sourceFile);
    let names;
    if (sourceSymbol) {
        names = typeChecker.getExportsOfModule(sourceSymbol).map(s => {
            if (s.flags & ts.SymbolFlags.Alias) {
                return typeChecker.getAliasedSymbol(s).name;
            }
            return s.name;
        });
    }
    else {
        names = [];
    }
    cachedNames.set(sourceFile, names);
    return names;
}
